"use client"
import {
  ColumnDef,
  flexRender,
  getCoreRowModel,
  getPaginationRowModel,
  getSortedRowModel,
  PaginationState,
  SortingState,
  useReactTable
} from '@tanstack/react-table';
import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from "../ui/table"
import { Button } from '../ui/button';
import { Badge } from '../ui/badge'
import { ArrowUpDown, Clock, Download, Trash, TriangleAlert } from 'lucide-react';
import { Dispatch, SetStateAction, useMemo, useReducer, useState } from 'react';
import Link from 'next/link';
import { IFile } from '../../types/file';
import { keepPreviousData, useQuery } from '@tanstack/react-query';
import { FilesApi } from 'apps/frontend/src/requests/files';
import { Alert, AlertDescription, AlertTitle } from '../ui/alert';
import { LoadingSpinner } from 'apps/frontend/src/components/loading-spinner';
import { Accordion, AccordionContent, AccordionItem, AccordionTrigger } from '../ui/accordion';
import {
  ISearchBarResult,
  SearchBar
} from 'apps/frontend/src/components/tables/search-bar';
import {
  TablePagination
} from 'apps/frontend/src/components/tables/pagination';

function ContextButtonForFile() {
  return (
    <div className={"scale-75"}>
      <Button variant={"destructive"}>
        <Trash />
      </Button>
    </div>
  );
}

export const filesTableColumns: ColumnDef<IFile>[] = [
  {
    accessorKey: "fileName",
    header: ({ column }) => (
      <div className={"flex justify-center items-center"}>
        Nom du fichier
      </div>
    ),
  },
  {
    accessorKey: "uploadedBy",
    header: ({ column }) => (
      <div className={"flex justify-center items-center"}>
        Autheur(s)
      </div>
    ),
  },
  {
    accessorKey: "uploadedAt",
    header: ({ column }) => (
      <Button
        variant="ghost"
        className={"flex w-full"}
        onClick={() => column.toggleSorting(column.getIsSorted() === "asc")}
      >
        Ajouté le
        <ArrowUpDown className="ml-2 h-4 w-4" />
      </Button>
    ),
    cell: ({ row }) => {
      const date = new Date(row.getValue("uploadedAt"));
      const formatted = `${date.getDate()}/${date.getMonth() + 1}/${date.getFullYear()} à ${date.getHours()}:${date.getMinutes()}`;
      return (
        <div className={"flex justify-center items-center"}>
          <Badge variant="outline" className={"gap-1 flex w-fit items-center"}>
            <Clock className={"w-4 h-4"} />
            <p className={"font-light"}>
              {formatted}
            </p>
          </Badge>
        </div>
      );
    },
  },
  {
    accessorKey: "extension",
    header: ({ column }) => (
      <div className={"flex justify-center items-center"}>
        Extension du fichier
      </div>
    ),
    cell: ({ row }) => {
      const extension = row.getValue("extension") as string;
      return (
        <div className={"flex justify-center items-center"}>
          <code className={"bg-gray-300 p-1 px-2 rounded-full"}>{extension}</code>
        </div>
      );
    },
  },
  {
    id: "actions",
    header: ({ column }) => (
      <div className={"flex justify-center items-center"}>
        Actions
      </div>
    ),
    cell: ({ row }) => {
      const file = row.original;
      return (
        <div className={"flex gap"}>
          <Button variant={"ghost"} asChild>
            <Link href={`http://localhost:3333/api/files/${file.uuid}`}>
              <Download />
              Télécharger
            </Link>
          </Button>
          <ContextButtonForFile />
        </div>
      );
    },
  },
];

interface DataTableProps<TData, TValue> {
  columns: ColumnDef<TData, TValue>[]
  searchField: ISearchBarResult
}

export function FilesDataTable<TData, TValue>({
                                                columns,
                                                searchField
                                              }: DataTableProps<TData, TValue>) {
  const rerender = useReducer(() => ({}), {})[1];
  const [sorting, setSorting] = useState<SortingState>([]);
  const [data, setData] = useState<TData[]>([]);
  const [rowsCount, setRowsCount] = useState(0);
  const [pagination, setPagination] = useState<PaginationState>({
    pageIndex: 0,
    pageSize: 10,
  });

  const queryResult = useQuery({
    queryKey: ['files', pagination, searchField],
    queryFn: async () => {
      const response = await FilesApi.get.files(pagination, searchField.value);
      setRowsCount(response.count);
      setData(response.data as TData[]);
      return response.data;
    },
    staleTime: 500,
    placeholderData: keepPreviousData,
  });

  const { isPending, isError, error, isFetching, isPlaceholderData } = queryResult;

  const table = useReactTable({
    data,
    columns,
    pageCount: Math.ceil(rowsCount / pagination.pageSize),
    getCoreRowModel: getCoreRowModel(),
    getPaginationRowModel: getPaginationRowModel(),
    manualPagination: true,
    onSortingChange: setSorting,
    getSortedRowModel: getSortedRowModel(),
    state: {
      sorting,
      pagination,
    },
  });


  if (isError) {

    return (
      <Alert variant="destructive">
        <TriangleAlert />
        <AlertTitle>Erreur</AlertTitle>
        <AlertDescription>
          {error.message}
          <Accordion type="single" collapsible>
            <AccordionItem value="item-1">
              <AccordionTrigger>Stack trace</AccordionTrigger>
              <AccordionContent>
                <code className="text-sm font-mono border-l-2 border-l-destructive h-fit">{error.stack}</code>
              </AccordionContent>
            </AccordionItem>
          </Accordion>
        </AlertDescription>
      </Alert>
    );
  }

  return (
    <div className="w-full">
      {isPending && isFetching && isPlaceholderData ? <LoadingSpinner/> : <Table>
        <TableHeader>
          {table.getHeaderGroups().map((headerGroup) => (
            <TableRow key={headerGroup.id}>
              {headerGroup.headers.map((header) => (
                <TableHead key={header.id}>
                  {header.isPlaceholder ? null : flexRender(
                    header.column.columnDef.header,
                    header.getContext()
                  )}
                </TableHead>
              ))}
            </TableRow>
          ))}
        </TableHeader>
        <TableBody>
          {table.getRowModel().rows?.length ? (
            table.getRowModel().rows.map((row) => (
              <TableRow key={row.id} data-state={row.getIsSelected() && "selected"}>
                {row.getVisibleCells().map((cell) => (
                  <TableCell key={cell.id}>
                    {flexRender(cell.column.columnDef.cell, cell.getContext())}
                  </TableCell>
                ))}
              </TableRow>
            ))
          ) : (
            <TableRow>
              <TableCell colSpan={columns.length} className="h-24 text-center">
                Auccun résultat
              </TableCell>
            </TableRow>
          )}
        </TableBody>
      </Table>}
      <div className={"flex w-full justify-end items-center"}>
        <TablePagination rowsCount={rowsCount} pagination={pagination} setPagination={setPagination}/>
      </div>
    </div>
  );
}
