import { PaginationState } from "@tanstack/react-table";
import { IFile, IFilesResponse } from "../types/file";
import axios from 'axios';

const API_URL = "http://localhost:3333/";
const PAGE_LIMIT = 10;

function getFullRoute(route: string) {
	return API_URL + route;
}

/**
 * Fetches a list of files from the server based on pagination and an optional search field.
 *
 * @param {PaginationState} pagination - The current state of pagination, including page size and page index.
 * @param {string} [searchField] - An optional field used to filter the files by a search term.
 * @return {Promise<IFilesResponse>} A promise that resolves to an IFilesResponse object containing the list of files.
 */
async function getFiles(
	pagination: PaginationState,
	searchField?: string,
): Promise<IFilesResponse> {
	const offset = pagination.pageSize * pagination.pageIndex;
	const res = await fetch(
		`${getFullRoute("api/files/find")}?limit=${PAGE_LIMIT}&offset=${offset}${searchField ? `&search=${searchField}` : "&search="}`,
	);
	return res.json();
}

/**
 * Uploads a file to the server with specified metadata.
 *
 * @param {File} file - The file to be uploaded.
 * @param {string} fileName - The name to be assigned to the uploaded file.
 * @param {string} uploadedBy - The identifier for the user uploading the file.
 * @param {Array<string>} machineIds - An array of machine IDs associated with the file.
 * @param {string|null} groupId - The group ID for grouping files, if applicable.
 * @param {boolean} [isDocumentation=false] - Flag indicating if the file is documentation.
 * @param {boolean} [isRestricted=false] - Flag indicating if the file is restricted.
 * @return {Promise<Object>} A promise resolving to the server's response data.
 */
async function uploadFile(
	file: File,
	fileName: string,
	uploadedBy: string,
	machineIds: Array<string>,
	groupId: string|null = null,
	isDocumentation = false,
	isRestricted = false
) {

	const headers = {
		"Content-Type": "application/octet-stream",
		file_name: fileName,
		uploaded_by: uploadedBy,
		//machine_id: machineIds.join(","), // Assuming machine IDs should be a comma-separated string
		machine_id: machineIds,
		is_documentation: isDocumentation.toString(),
		is_restricted: isRestricted.toString(),
	};

	if (groupId) {
		// @ts-ignore
		headers["group_id"] = groupId;
	}
	try {
		const response = await axios.post(getFullRoute("api/files/new"), file, { headers });
		return response.data;
	} catch (error) {
		console.error("Error uploading file:", error);
		throw error;
	}

}

export const FilesApi = {
	get: {
		files: getFiles,
	},
	post: {
		upload: uploadFile,
	}
};
